# Generated by Django 3.2.15 on 2022-10-07 07:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nautobot_cable_utils', '0003_rename_tenant_cabletemplate_owner'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cabletemplate',
            old_name='cable_number',
            new_name='name',
        ),
    ]
