# Generated by Django 3.2.25 on 2024-06-21 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0058_controller_data_migration'),
        ('tenancy', '0009_update_all_charfields_max_length_to_255'),
        ('nautobot_cable_utils', '0009_auto_20230802_0724'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='CableTemplate',
            new_name='CableInventoryItem',
        ),
    ]
