# Generated by Django 3.2.25 on 2024-06-21 14:09

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0058_controller_data_migration'),
        ('tenancy', '0009_update_all_charfields_max_length_to_255'),
        ('nautobot_cable_utils', '0010_rename_cabletemplate_cableinventoryitem'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cableinventoryitem',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_inventory_items', to='tenancy.tenant'),
        ),
        migrations.AlterField(
            model_name='cableinventoryitem',
            name='plug',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cable_inventory_items', to='nautobot_cable_utils.cableplug'),
        ),
        migrations.AlterField(
            model_name='cableinventoryitem',
            name='storage_rack',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_inventory_items', to='dcim.rack'),
        ),
        migrations.AlterField(
            model_name='cableinventoryitem',
            name='supplier',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_inventory_items', to='dcim.manufacturer'),
        ),
    ]
