{% if not object.cable %}
    <div class="btn-group">
        <button type="button" class="btn btn-primary dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            <span class="mdi mdi-plus-thick" aria-hidden="true"></span> Auto-Link <span class="caret"></span>
        </button>
        <ul class="dropdown-menu">
            <li><a href="{% url 'plugins:nautobot_cable_utils:cable_router_link' termination_a_type="interface" termination_a_id=object.id termination_b_type="interface" %}" >Interface</a></li>
            <li><a href="{% url 'plugins:nautobot_cable_utils:cable_router_link' termination_a_type="interface" termination_a_id=object.id termination_b_type="front-port" %}" >Front Port</a></li>
        </ul>
    </div>
{% endif %}