# Generated by Django 3.2.18 on 2023-03-24 10:55

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tenancy', '0002_auto_slug'),
        ('dcim', '0019_device_redundancy_group_data_migration'),
        ('nautobot_cable_utils', '0007_alter_cabletemplate_supplier'),
    ]

    operations = [
        migrations.AddField(
            model_name='cabletemplate',
            name='storage_rack',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_templates', to='dcim.rack'),
        ),
        migrations.AlterField(
            model_name='cabletemplate',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='cable_templates', to='tenancy.tenant'),
        ),
    ]
